import json 
import os 
import pkg_resources
import sys
import click

import configparser 

import utils.logger as logger

config = configparser.SafeConfigParser()


def get_config_path():
    homedir = os.environ.get('HOME', None)
    logger.debug("Home Directory: " + homedir)
    if not homedir:
        logger.log_r("Home Directory Not found!! Set Envirnoment `HOME` ")
        exit()
    logger.debug("Home Directory : " + homedir)
    config_file = os.path.join(homedir + "/.dokr/config")
    logger.debug("Config File Location : " + config_file)
    if not os.path.exists(config_file):
        logger.log_r("ERROR: No Config file present")
        try:
            os.makedirs(os.path.dirname(config_file))
        except OSError as exc:  # Guard against race condition
            logger.log_r("Directory found! but not config file")
            
        logger.debug("Creating config file")
        file = open(config_file, "w") 
        file.write("[default]") 
        file.close() 
        
    logger.debug(config.read(config_file))
    return config_file;


config_file = get_config_path();

 
def ask_for_new_profile():
    add_profile('deepak')
    print(get_env('fabhotelier', 'ecr_repository'))
    print(set_env('fabhotelier', 'ecr_repository', 'My'))
    print(set_env('deepak', 'ecr_repository', 'My'))


def set_env(profile, key , value):
     config.set(profile, key, value)
     write_config()


def add_profile(profile):
    if config.has_section(profile):
        logger.log_r(profile + " Section already exists!!")
        return
    
    config.add_section(profile)
    write_config()


def write_config():
    with open(config_file, 'w') as configfile:
        config.write(configfile)


def get_env(profile, key):
    logger.debug("Searching in profile : " + profile)
    logger.debug("Searching in key" + key)
    if config.has_option(profile, key):
        return config.get(profile, key)
    logger.debug("Not found in current profile")
    if config.has_option('default', key):
        return config.get('default', key)
    
    logger.debug("No Value Found in DEAFULT SECTION as well")
    logger.log_r('Value not found in [Default Profile] use `dokr configure`comamnd')
    exit()

    
def get_profiles():
    return config.sections()


def log_config():
    level_input = 'DEBUG';
    if level_input == 'DEBUG':
        print(level_input)
        logging.basicConfig(level=logging.DEBUG)


def find_version():
    version = pkg_resources.require("dokr")[0].version  
    print (version)      




import os
import shlex
import subprocess
import sys

import commons.constant as const
import utils.logger as logger


def cmd_exec(str):
    logger.debug("\n------------------------ Executing Command: Start ------------------------")
    logger.debug("\n$>>" + str)    
    output = os.popen(str).read().strip()
    logger.debug("\n$>>" + output)
    logger.debug("\n------------------------ Executing Command: END ------------------------")
    return output


def join_me(stringList):
    return "".join(string for string in stringList)


def running_cmd(cmd):
    logger.log_c("Running command: " , cmd)
    subprocess.call(cmd.split(" "))


def get_system_type():
    platform = sys.platform;
    logger.debug("Platform : " + platform)
    system_name = "NA";
    if platform == "linux" or platform == "linux2":
       system_name = const.LINUX_OS
    elif platform == "darwin":
        system_name = const.MAC_OS
    elif platform == "win32":
        system_name = const.WINDOWS_OS
        
    logger.debug("System Name : " + system_name)
    return system_name;


def helper():
    print('\n------------------------ Command Options ------------------------')
    print('\ndokr clean searchPattern # delete old images matching pattern')
    print('\ndokr clean-all searchPattern # clean docker')
    print('\ndokr lecs # ECS login')
    print('\ndokr tag searchPattern tagName # add a tag *tagName*on a image matching *searchPattern* ')
    print('\ndokr push searchPattern  # push all images matching the pattern')
    print('\ndokr ip searchPattern  # push all images matching the pattern')
    print('\ndokr push searchPattern  # push all images matching the pattern')
    print("\ndokr ip searchPattern # Find Ip from your aws machine")
    print("\ndokr log # For cluster->service->task logging. Provided ecs-cli is installed")
    print('\n')



import sys
import os
import json
import utils.helper as utils
import commons.config as config
import commons.constant as const
import utils.logger as logger
import click




def get_latest_task_definations(family_prefix):
    task_definations = utils.cmd_exec("aws ecs list-task-definitions --family-prefix " + family_prefix);
    return json.loads(task_definations)['taskDefinitionArns'][-1]


def get_task_defination_json(task_def_name):
    task_defination_json = utils.cmd_exec("aws ecs describe-task-definition --task-definition " + task_def_name);
    return json.loads(task_defination_json)


def update_task_image_version(task_def_json, version_number):
    imageName = task_def_json['taskDefinition']['containerDefinitions'][0]['image'];
    imageName = imageName.split(':')[0] + ':' + version_number
    task_def_json['taskDefinition']['containerDefinitions'][0]['image'] = imageName;
    logger.debug("\nUpdated Task Defination : " + json.dumps(task_def_json, indent=4))
    return task_def_json['taskDefinition'];


def cluster_list():
    clusters = utils.cmd_exec("aws ecs list-clusters")
    clusters = json.loads(clusters)
    return clusters['clusterArns']


def services_in_cluster(service_name):
    services = utils.cmd_exec("aws ecs list-services --cluster " + service_name)
    services = json.loads(services)
    return services['serviceArns']


def tasks_in_cluster(cluster_name, service_name):
    tasks = utils.cmd_exec("aws ecs list-tasks --cluster " + cluster_name + " --service-name " + service_name)
    tasks = json.loads(tasks)
    return tasks['taskArns']


def get_index_input(list, query):
    if len(list) == 0:
        logger.log_c("\n" + query + " list  size is 0 !! Exiting program\n");
        exit()  
    logger.log_c("\nSelect one " + query + " from the list: \n")
    index = 1
    for item in list:
        logger.log_r(str(index) + ". " + item.split('/')[1])
        index = index + 1; 
    
    index = click.prompt("\nYour Options? :" , default=1, type=int)
    logger.log_c("\nSelected Input for " + query + " : " + list[index - 1].split('/')[1]);
    return list[index - 1].split('/')[1]


def check_aws_cli():
    system_name = utils.get_system_type();
    path = const.ECS_CLI_PATH_MAP[system_name]
    exists = os.path.isfile(path)
    
    if exists == False:
        print("Checked path : " + path + " on " + system_name)
        print(const.ECS_CLI_INSTRUCTIONS)


def clean_task_defination(task_json):
    del task_json['status']
    del task_json['requiresAttributes']
    del task_json['taskDefinitionArn']
    del task_json['compatibilities']
    del task_json['revision']
    return task_json


def register_task(task_family, task_def_json):
    utils.cmd_exec("aws ecs register-task-definition --family " + task_family + " --cli-input-json \"" + json.dumps(task_def_json).replace('"', '\\"').replace('\n', '\\n') + "\"")


def get_task_family(cluster_name, service_name):
    service_json = utils.cmd_exec("aws ecs describe-services --cluster " + cluster_name + " --service " + service_name)
    service_json = json.loads(service_json)
    return service_json['services'][0]['deployments'][0]['taskDefinition'].split("/")[1]


# For ECS login
def login_ecs():
    logger.log_y("Logging into ECS")
    login = utils.cmd_exec("aws ecr get-login   | sed  's/-e none//g'")
    utils.cmd_exec(login)
    logger.log_g("Login successful")

# Print the running log
def ecs_log():
    check_aws_cli()
    
    clusters = cluster_list()
    cluster_name = get_index_input(clusters, "Cluster");
    
    logger.log_c("Cluster name :" , cluster_name)
    
    services = services_in_cluster(cluster_name)
    service_name = get_index_input(services, "Service");
    logger.log_c("Service Name :" , service_name)
    
    tasks = tasks_in_cluster(cluster_name, service_name)
    task_name = get_index_input(tasks, "Task");
    
    utils.running_cmd("ecs-cli logs -c " + cluster_name + " --task-id " + task_name + " --follow")

    
# Deploy an image on an ECS cluster
def deploy(cluster_name, service_name, image_version):
    task_family = get_task_family(cluster_name, service_name)
    logger.log_c("Deploying Task Family : " , task_family)
    logger.log_c("Cluster Name : " , cluster_name)
    logger.log_c("Service Name : " , service_name)
    task_family = task_family.split(":")[0]
    last_task_def_name = get_latest_task_definations(task_family)
    task_def_json = get_task_defination_json(last_task_def_name)
    task_def_json = update_task_image_version(task_def_json, image_version)
    older_revision = json.dumps(task_def_json['revision'])
    logger.log_c("Last Task Version Deployed : " , older_revision)
    new_revision = str(int(older_revision) + 1);
    logger.log_c("Deploying new Task version : " , new_revision)
    task_def_json = clean_task_defination(task_def_json)
    logger.debug('Updated JSON of Task definations : ' + json.dumps(task_def_json, indent=4))
    register_task(task_family, task_def_json)
    utils.cmd_exec("aws ecs update-service --cluster " + cluster_name + " --service " + service_name + " --task-definition " + task_family + ":" + new_revision)
    logger.log_c("Successfully deployed task version " , new_revision)


# Find the ip of an instance
def find_ip(instanceToSearch):
    logger.log_c("Fetching instance list")
    ec2_instances = utils.cmd_exec("aws ec2 describe-instances")
    ec2_instances = json.loads(ec2_instances)
    found_instance = {};
    logger.debug("Searching the pattern : " + instanceToSearch)
    reservations = ec2_instances['Reservations'];
    for reservation in reservations:
        instances = reservation['Instances']
        for instance in instances:
            tags = instance['Tags'];
            for tag  in tags:
                tagKey = json.dumps(tag['Key']);
                tagValue = json.dumps(tag['Value']);
                if instanceToSearch in tagValue :
                    logger.debug("Found instance  : " + tagValue)
                    found_instance = instance;
                    break
    
    ip_address = found_instance['NetworkInterfaces'][0]['Association']['PublicIp'];
    logger.log_c("Ip Address of " + instanceToSearch + ": " , ip_address)



import sys
import os
import utils.helper as utils;

import utils.logger as logger;
import commons.config as config
import click


def add_build_tag(search_pattern, tag_name):
    logger.log_c("\nSearching pattern: " , search_pattern)
    logger.log_c("Adding tag : " + tag_name)
    image_name = utils.cmd_exec("docker images | cut -d ' '  -f1 | grep " + search_pattern + " | head -1");
    logger.log_y("Found this image name from the given pattern : " + image_name)
    add_tag_command = utils.join_me(['docker tag ' , image_name , ' ' , image_name , ':', tag_name ]);
    utils.cmd_exec(add_tag_command);
    logger.log_y('Added Tag : ' + tag_name + ' into the images\n')
    images_list = utils.cmd_exec('docker images')
    logger.log_c("Updated Images list :\n")
    logger.log(images_list)

  
def push_image(search_pattern):
    logger.log_c("\nSearching images matching this pattern : " + search_pattern)
    image_name = utils.cmd_exec("docker images | cut -d ' '  -f1 | grep " + search_pattern + " | head -1");
    logger.log_y("\nFound this image name from the given pattern : " + image_name)
    
    if image_name == '':
        logger.log_r("No such found to push !!! Exiting !!!")
        exit();
    image_list_str = "docker image inspect -f '{{join .RepoTags \"\\n\" }}' " + image_name;
    logger.debug("\nFinding images tags")
    image_list = utils.cmd_exec(image_list_str)
    logger.debug("\nFound Following images : \n\n" + image_list)
    tag_list = utils.cmd_exec(image_list_str).split("\n")
    logger.log_c('\nTotal Tags found : ' , str(len(tag_list)))
        
    for tag in tag_list:
        logger.log_c('\n--------------------------Pushing image :' + tag + '-----------------------------------')
        utils.cmd_exec('docker push ' + tag)


def clean_up(search_pattern):
    logger.log_c('\nCleaning Old Images  matching pattern : ' , search_pattern)
    logger.debug('\n----------  Cleaning Old Images ---- \n')
    utils.cmd_exec("docker images -a | grep " + search_pattern + " | awk '{print $3}' | xargs docker rmi -f")
    logger.log_g("\nCleaned images successfully")


def clean_all():
    logger.log_c('\nCleaning docker system')
    logger.log_y(utils.cmd_exec("docker system prune -a -f"))


def add_default_profile():
    registry = click.prompt(logger.style('\n[default] Your docker registry '), type=str)
    dir_mapping = click.prompt(logger.style('\n[default] Your default directory mapping '), type=str)
    env_vars = click.prompt(logger.style('\n[default] Environment vars(eg. ENV_NAME=dev)  '), type=str)
    config.set_env('default', 'docker_registry', registry)
    config.set_env('default', 'vol_mapping', dir_mapping)
    config.set_env('default', 'env_vars', env_vars)


def input_config_vars(profile_name, key_name, is_default_exist):
    if is_default_exist :
        default_value = config.get_env('default', key_name)
        user_input = click.prompt(logger.style('\nValue of ' + key_name), default=default_value, type=str)
    else:
        user_input = click.prompt(logger.style('\nYour ' + key_name), type=str)
    
    config.set_env(profile_name, key_name, user_input)
    

def add_profile(profile_name):
    config.add_profile(profile_name)
    config.set_env(profile_name, 'app_name', profile_name)
    input_config_vars(profile_name, 'docker_registry', True)
    input_config_vars(profile_name, 'vol_mapping', True)
    input_config_vars(profile_name, 'port_mapping', False)
    input_config_vars(profile_name, 'env_vars', True)

    
def run_profile(profile_name, tag_name):
    registry = config.get_env(profile_name, 'docker_registry')
    vol_mapping = config.get_env(profile_name, 'vol_mapping')
    port_mapping = config.get_env(profile_name, 'port_mapping')
    env_vars = config.get_env(profile_name, 'env_vars')
    
    command = "docker run -d "
    command += '--name ' + profile_name
    command += ' -p ' + port_mapping
    command += ' -v ' + vol_mapping
    command += ' -e ' + env_vars
    tag = 'latest'
    if tag_name == '':
        logger.log_bl('\nGetting top 5 available tags of ' + profile_name + ' from Amazon ECR registry...')
        tag_list = utils.cmd_exec("aws ecr describe-images --repository-name " + profile_name + " --output text --query 'sort_by(imageDetails,& imagePushedAt)[*].imageTags[*]' | tr '\t' '\n' | tail -5")
        tag_list = list(reversed(tag_list.split('\n')))
        tag = click.prompt(logger.style('\nSelect an tag to deploy: ?'), type=click.Choice(tag_list))
    logger.log_g('\nYou have selected tag: [ ' + tag + ' ]  for your application')
    command += ' ' + registry + '/' + profile_name + ':' + tag
    logger.debug('final command : ' + command)
    logger.log_y('\nKilling old container if exist')
    utils.cmd_exec('docker rm -f ' + profile_name)
    utils.cmd_exec(command)
    logger.log_g("\nSuccessfully started " + profile_name + " application . Please check logs using: ")
    logger.log_cy("\n                 docker logs -f " + profile_name + "                          \n")

    
def run_all():  
    profiles = config.get_profiles() 
    for profile in profiles :
        if profile == 'default' :
            continue
        logger.log_g("\n******************** Running Profile " + profile + " ********************")
        run_profile(profile, '')



import sys
import os
import utils.helper as utils
import utils.http_client as request
import json
import commons.config as config
import commons.constant as constant


def get_slack_url():
    if constant.SLACK_URI not in os.environ:
        uri = raw_input("What your slack URL ?")
        print ("Your URI is : " + uri)
        config.set_env(constant.SLACK_URI, "https://hooks.slack.com/services/" + uri)
    return config.get_env(constant.SLACK_URI);

def test_machine():
    url = get_slack_url();
    usage = utils.cmd_exec("docker exec -it fe0df5cdc63f  vmstat -s | grep 'memory'")
    message = raw_input("Enter the message you want to send : ")
    
    data = {
        'text': '>>> This is `sent` \n from *dokr* plugin \n>' + message + ' \n Memory Left on your device : ' + usage 
    }
    request.post(url, data)



import requests 
# importing the requests library 
import json

# For GET requests
def get(URL, PARAMS):
    # sending get request and saving the response as response object 

    r = requests.get(url=URL, params=PARAMS) 
    # extracting data in json format 
    return r.json() 

# For POST requests
def post(URL, payload):
    # sending get request and saving the response as response object 
    print (URL)
    print (payload)
    headerss = {'content-type': 'application/json'}
    r = requests.post(url=URL, data=json.dumps(payload), headers=headerss)
    print("Status code is :" , r.status_code)
    print("Response text is :" + r.text)
    return r;


import logging

SLACK_URI = 'DOKR_SLACK_URI'

LOGGING_LEVEL = {
    'DEBUG' : logging.DEBUG,
    'INFO' : logging.INFO
    }
# # ECS CLI  contants

MAC_OS = 'mac'
LINUX_OS = 'linux'

ECS_CLI_INSTRUCTIONS = """ [MAC]:\n 
        sudo curl -o /usr/local/bin/ecs-cli https://s3.amazonaws.com/amazon-ecs-cli/ecs-cli-linux-amd64-latest\n 
        chmod +x /usr/local/bin/ecs-cli
        
        -------   for LINUX ---------\n
        sudo curl -o /usr/local/bin/ecs-cli https://s3.amazonaws.com/amazon-ecs-cli/ecs-cli-linux-amd64-latest \n 
        chmod +x /usr/local/bin/ecs-cli
        
        further steps:
        follow readme.md
        https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ECS_CLI_installation.html
        """

ECS_CLI_PATH_MAP = {
    'mac' : '/usr/local/bin/ecs-cli',
    'linux' : '/usr/bin/ecs-cli'
    }



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='dokr',
    version='1.7',
    scripts=['dokr'] ,
    author="Deepak Kumar",
    author_email="deepak.kumar.iet@gmail.com",
    description="A Docker and AWS utility package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/javatechy/dokr",
    packages=setuptools.find_packages(),
    py_modules=['ecs_helper', 'docker_helper', 'slack_api'],
    install_requires=[
        'requests', 'click', 'configparser'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
 )
